/*       ͻ
             TITLE:  RTL8019AS ethernet routines for Microchip      
                     PIC18 series and Hitech PICC18 v.8.35PL3 port 
                     of Adam Dunkels' uIP v0.9                     
          FILENAME:  ne2000.c                                      
          REVISION:  VER 0.0                                       
          REV.DATE:  21-07-05                                      
           ARCHIVE:                                                
            AUTHOR:  Copyright (c) 2005, Engin Karabulut.          
            Refer to National Semiconductor DP8390 Datasheet       
         ͼ     */

/*  Ŀ
      THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS   
      OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED   
      WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE  
      ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY     
      DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL  
      DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   
      GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS       
      INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,        
      WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           
      NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS  
      SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.        
      */

#include <pic18.h>
#include "ne2000.h"
#include "delay.h"
#include "uip.h"


////////////////////////////////////////////////////////////////////////////////
#define adresport PORTA
#define dataport  PORTD
#define IOR		RE0
#define	IOW		RE1
#define RESETNIC RE2

// NIC Page 0 read register assignments

#define CR		0x00						// Command
#define CLDA0	0x01						// Current Local DMA Address 0
#define CLDA1	0x02						// Current Local DMA Address 1
#define BNDRY	0x03						// Boundary Pointer
#define TSR		0x04						// Transmit Status Register
#define NCR		0x05						// Number of Collisions Register
#define FIFO	0x06						// FIFO
#define ISR		0x07						// Interupt Status Register
#define CRDA0	0x08						// Current Remote DMA Address 0
#define CRDA1	0x09						// Current Remote DMA Address 1
#define RES1	0x0A						// Reserved
#define RES2	0x0B						// Reserved
#define	RSR		0x0C						// Receive Status Register
#define CNTR0	0x0D						// Tally Counter 0 (Frame Alignment Errors)
#define CNTR1	0x0E						// Tally Counter 1 (CRC Errors)
#define CNTR2	0x0F						// Tally Counter 2 (Missed Packet Errors)

// NIC Page 0 write register assignments

#define PSTART	0x01						// Page Start Register
#define PSTOP	0x02						// Page Stop Register
#define	TPSR	0x04						// Transmit Page Start Address
#define	TBCR0	0x05						// Transmit Byte Count Register 0
#define TBCR1	0x06						// Transmit Byte Count Register 1
#define RSAR0	0x08						// Remote Start Address Register 0
#define RSAR1	0x09						// Remote Start Address Register 1
#define RBCR0	0x0A						// Remote Byte Count Register 0
#define RBCR1	0x0B						// Remote Byte Count Register 1
#define RCR		0x0C						// Receive Configuration Register
#define TCR		0x0D						// Transmit Configuration Register
#define DCR		0x0E						// Data Configuration Register
#define IMR		0x0F						// Interrupt Mask Register

// NIC Page 1 register assignments

#define PAR0	0x01						// Physical Address Register 0
#define PAR1	0x02						// Physical Address Register 1
#define PAR2	0x03						// Physical Address Register 2
#define PAR3	0x04						// Physical Address Register 3
#define PAR4	0x05						// Physical Address Register 4
#define PAR5	0x06						// Physical Address Register 5
#define CURR	0x07						// Current Page Register
#define MAR0	0x08						// Multicast Address Register 0
#define MAR1	0x09						// Multicast Address Register 1
#define MAR2	0x0A						// Multicast Address Register 2
#define MAR3	0x0B						// Multicast Address Register 3
#define MAR4	0x0C						// Multicast Address Register 4
#define MAR5	0x0D						// Multicast Address Register 5
#define MAR6	0x0E						// Multicast Address Register 6
#define MAR7	0x0F						// Multicast Address Register 7


// NIC Other registers

#define NIC_DATA	0x10					// Data Register for I/O port mode
#define	NIC_RESET	0x18					// Reset Register
//configuration registers initial values
#define RTL8019_RCR_INIT      0x04  //accept broadcast packets and packets destined to this mac
#define RTL8019_IMR_INIT      0xFF  //enable all interrupts
#define RTL8019_RXSTOP_INIT   0x60  //last page of buffer in rtl8019 device
#define RTL8019_TXSTART_INIT  0x40  //start page for tx buffer
#define RTL8019_RXSTART_INIT  0x46  //end page for tx buffer (total pages = 6, total bytes = 6 * 256)
#define RTL8019_DCR_INIT      0x40  //byte wide dma,  loopback, send packet command executed, fifo thresh select bit 1
#define RTL8019_TCR_INIT      0x00  //enable crc on tx and rx, normal operation (no loopback)
#define ETHERNET_MIN_PACKET_LENGTH  60   //rtl8019 cannot send packets smaller than this
/////////////////////////////////////////////////////////////////////////////////
unsigned char my_mac[6] = {0x00,0x02,0x44,0x4A,0x93,0xCE};  //my mac number
unsigned char next_page;            //pointer to next page of rx buffer
unsigned int current_rx_address;   //pointer to next byte to get from rx buffer
/////////////////////////////////////////////////////////////////////////////////

void outport(unsigned char address,unsigned char data)
{

	TRISD = 0x00;						// Switch the data pins to output
    adresport = address;			// Put the register address on the address pins							
    dataport = data;			   // Put the data on the bus
    IOW = 0;						// Activate the I/O write request
	IOW = 1;					// Deactivate the I/O write request
	TRISD = 0xFF;				// Reset data port to input
}

/////////////////////////////////////////////////////////////////////////////////

unsigned char inport(unsigned char address)
{
	unsigned char data;
	TRISD = 0xFF;
    adresport = address;					// Output the correct register address								
	IOR = 0;         						// Activate the I/O read request
	DelayBigUs(1);
    data = dataport;						// Input data from the bus
	IOR = 1;				        		// Deactivate the I/O read request
	return data;
}

////////////////////////////////////////////////////////////////////////////////
void stop(void)
{
  while(1)
  {
  #asm
  nop
  #endasm
  }
}
////////////////////////////////////////////////////////////////////////////////
void hard_reset(void)
{
 RESETNIC =1;
 DelayMs(10);
 RESETNIC =0;
}

////////////////////////////////////////////////////////////////////////////////
void nic_init(void)
{
   int i;
   
   outport(CR , 0x21); //stop rtl8019, abort dma
   outport(DCR, RTL8019_DCR_INIT);
   outport(RBCR0,0x00); //clear byte count registers
   outport(RBCR1,0x00);
   outport(RCR, RTL8019_RCR_INIT);   //packets with broadcast destination address are accepted
   outport(TCR,0x02);   //set to internal loopback
   outport(TPSR, RTL8019_TXSTART_INIT);
   outport(BNDRY, RTL8019_RXSTART_INIT);
   outport(PSTART, RTL8019_RXSTART_INIT);
   outport(PSTOP, RTL8019_RXSTOP_INIT);
   outport(ISR, 0xFF);  //clear isrs, very important!
   outport(IMR, RTL8019_IMR_INIT);
   outport(CR, 0x61);   //set to page 1
   outport(CURR, RTL8019_RXSTART_INIT);
   outport(PAR0, my_mac[0]);  //set ethernet mac address
   outport(PAR1, my_mac[1]);
   outport(PAR2, my_mac[2]);
   outport(PAR3, my_mac[3]);
   outport(PAR4, my_mac[4]);
   outport(PAR5, my_mac[5]);
   for (i=0;i<8;i++) {
      outport(MAR0+i, 0xFF);
   }
   outport(CR, 0x22); // start the NIC
   outport(DCR, 0x48);
   outport(TCR,0x00);   //set to normal op.
   
}
////////////////////////////////////////////////////////////////////////////////
void rtl8019_overrun(void)
{
   unsigned char data;

   data = inport(ISR);
   if (data & 0x10) {
      outport(CR, 0x21);   //stop rtl8019, abort transmissions
      DelayMs(2);
      outport(RBCR0, 0x00);   //clear byte count registers
      outport(RBCR1, 0x00);
      outport(TCR, 0x02);  //set to internal loopback
      outport(CR, 0x22);   //set to page 0
      outport(BNDRY, RTL8019_RXSTART_INIT);   //last receive buffer
      outport(CR, 0x62);   //set page to 1
      outport(CURR, RTL8019_RXSTART_INIT);   //points to first page address of the first rx buffer page to be used for packet reception
      outport(CR, 0x22);   //set to page 0
      outport(ISR, 0x10);  //overflow (OVW) (ISR bits are cleared by setting to 1)
      outport(TCR, RTL8019_TCR_INIT);  //transmit control register
      outport(ISR, 0xFF);
   }
}
////////////////////////////////////////////////////////////////////////////////

unsigned int nic_begin_packet_rx(void)
{
   unsigned char i, bnry, header[4];
   unsigned int len;

   rtl8019_overrun();  //check rx overrun

   outport(CR,0x62);
   i = inport(CURR); //get current page pointer

   outport(CR,0x22);
   bnry = inport(BNDRY);   //get last page pointer we have read
   //stop(); 
   if (bnry == i) {return(0);} //if pointers are same there is no new packet

   outport(ISR,0x01);  //clear rx interrupt flag

   if( (bnry >= RTL8019_RXSTOP_INIT) || (bnry < RTL8019_RXSTART_INIT) )   //make sure pointer is valid
   {
      outport(BNDRY, RTL8019_RXSTART_INIT);
      outport(CR, 0x62);
      outport(CURR, RTL8019_RXSTART_INIT);
      outport(CR, 0x22);
      return(0);
   }
   //lcd_putch(bnry);
   outport(RBCR0, 0x04);   //set dma to get 4 bytes
   outport(RBCR1, 0x00);
   outport(RSAR0, 0);   //set address to begining of packet
   outport(RSAR1, bnry);
   outport(CR, 0x0A);   //set dma to read
   for(i=0;i<4;i++) {
      header[i] = inport(NIC_DATA);   //read 4byte header
   }
   

   outport(CR, 0x22);   //end dma (it should have ended already, but just in case)
   for(i = 0; i <= 20; i++) 
   {
      if ( (inport(ISR))&0x40 ) {break;}  //wait until dma is completed
   }
   outport(ISR, 0x40);

   len = ((unsigned int)header[3])<<8;  //get len from header
   len = len + header[2];               //get len from header

   next_page = header[1];

   current_rx_address = (((unsigned int)bnry)<<8) + 4;

   if( (next_page >= RTL8019_RXSTOP_INIT) || (next_page < RTL8019_RXSTART_INIT) ) {
      return(0);   //dont continue if data is not in valid buffer
   }

   return (len-4);
}
////////////////////////////////////////////////////////////////////////////////
void nic_packet_get_data(unsigned int len)
{
   unsigned int i;
   unsigned char degis;
   outport(RBCR0, (unsigned char)len); //set dma read count
   outport(RBCR1, (unsigned char) ((len >>8)&0x00FF) );
   outport(RSAR0, (unsigned char)current_rx_address  );   //set dma read address
   outport(RSAR1, (unsigned char) ((current_rx_address >>8)&0x00FF));
   outport(CR, 0x0A);   //set dma to read
  
   
   for(i=0;i<len;i++)
   {
   *(uip_buf + i) = inport(NIC_DATA);
   }
   degis = uip_buf[12];
   uip_buf[12] = uip_buf[13];
   uip_buf[13] = degis; 
   outport(CR, 0x22);   //end dma (it should already have ended)
   for(i = 0; i <= 20; i++) {
      if ((inport(ISR)&0x40)) {break;} //wait until dma is completed
   }
   outport(ISR, 0x40);

   current_rx_address += len;   //set pointer to next byte in buffer to read for next read attempt

   if( current_rx_address >= 0x6000 )   //!
   current_rx_address = current_rx_address - (0x6000-0x4600) ;   //!
}
////////////////////////////////////////////////////////////////////////////////
//Ends packet reading initiated by nic_begin_packet_rx().  Free's rtl8019
// receive buffer of this packet.  Will throw away any un-read data of the packet.
void nic_end_packet_rx(void)
{
   unsigned char i;

   outport(CR, 0x22);   //end dma operation
   for(i = 0; i <= 20; i++) {
      if ((inport(ISR)&0x40)) {break;} //wait until dma is finished
   }
   outport(ISR, 0x40);

   outport(BNDRY, next_page);   //set to start of next packet
}

void remote_dma_setup(long int address)
{

	outport(CR, 0x22);						// Abort Remote DMA
	outport(RBCR0, 0xFF);					// Set maximum number of bytes to read/write
	outport(RBCR1, 0xFF);
	outport(RSAR0, address);
	outport(RSAR1, address>>8);
    outport(CR, 0x12);
}

void send_packet()  //ethernet packet send operation
{
    unsigned int i;
    unsigned char *ptr;
    ptr = uip_buf;  //pass uip_buf location to local ptr pointer  

    remote_dma_setup(0x40 << 8);
    for(i = 0; i < uip_len; i++)
    {
        if(i == 40 + UIP_LLH_LEN) 
        {
            ptr = (unsigned char *)uip_appdata; //above uip_buf[40] dedicated to application layer. look uip.c for details 
             
        }

        outport(NIC_DATA, *ptr++);  //send data to NIC's local ring buffer
    }
	if(uip_len<64) //check if the total length of ethernet packet is  bigger then 64, packets whose length is below 64 is ignored generally by remote host.
    {
      i=uip_len;
      for(;i<64;i++) 
      {outport(NIC_DATA,0); } //so we padded to 64 with zeros.
      uip_len = 64; 
    }
    outport(CR, 0x22);
	outport(TBCR0, uip_len ); //enter the total packet length
	outport(TBCR1, uip_len >> 8);
	outport(TPSR,0x40); //enter the starting location of send buffer
	outport(CR, 0x26);  //execute send action
    
}


